package com.albertoventurini.parsley.examples.graphml;

import com.albertoventurini.parsley.grammar.Grammar;
import com.albertoventurini.parsley.grammar.rules.Rule;

import java.util.function.BiFunction;
import java.util.function.Function;

import static com.albertoventurini.parsley.grammar.rules.Rules.*;

public class GraphMLGrammar extends Grammar {

    private final Rule comment = sequence(string("<!--"), until("-->")).as("comment");
    private final Rule comments = zeroOrMore(comment).setComment(true).as("comments");

    private final Rule xmlHeader = string("<?xml version='1.0' ?>").as("header");

    private final Rule attribute = sequence(
            takeWhile(c -> c != '=' && c != '>'),
            character('='),
            character('\''),
            takeWhile(c -> c != '\''),
            character('\'')).as("attribute");

    private final Function<String, Rule> tagFunc = (s) -> sequence(
            character('<'),
            string(s).as("tagName"),
            zeroOrMore(attribute).as("attributes"),
            character('>')).as("tag:" + s);

    private final Function<String, Rule> closingTagFunc = (s) -> sequence(
            string("</"),
            string(s),
            character('>')).as("closingTag:" + s);

    private final BiFunction<String, Rule, Rule> elementFunc = (tagName, content) -> sequence(
            tagFunc.apply(tagName).as("tag:" + tagName),
            content.as("content"),
            closingTagFunc.apply(tagName).as("closingTag:" + tagName));

    private final Rule data = elementFunc.apply("data", takeWhile(c -> c != '<')).as("dataElem");

    private final Rule node = elementFunc.apply("node", zeroOrMore(data)).as("nodeElem");

    private final Rule edge = elementFunc.apply("edge", zeroOrMore(data)).as("edgeElem");

    private final Rule graph = sequence(
            tagFunc.apply("graph"),
            oneOrMore(node).as("nodes"),
            oneOrMore(edge).as("edges"),
            closingTagFunc.apply("graph")).as("graphElem");

    private final Rule key = sequence(tagFunc.apply("key"), closingTagFunc.apply("key"))
            .as("keyElem");

    private final Rule graphml = sequence(
            tagFunc.apply("graphml"),
            oneOrMore(key).as("keys"),
            graph,
            closingTagFunc.apply("graphml")).as("graphmlElem");

    private final Rule graphmlFile = sequence(
            xmlHeader,
            comments.discard(),
            graphml).as("graphmlFile");

    @Override
    protected Rule startRule() {
        return graphmlFile;
    }

    @Override
    protected Rule commentRule() {
        return comments;
    }
}
